import React, { useMemo, useRef } from 'react';
import { Table, AugmentedNodeAttributes } from './components/Table';
import { SchemaAttribute } from '../../../schema';
import { VisualizationPropTypes, VISComponentType } from '../../types';
import Input from '@graphpolaris/shared/lib/components/inputs';
import { GraphMetaData } from '@graphpolaris/shared/lib/data-access/statistics';
import { SettingsContainer, SettingsHeader } from '@graphpolaris/shared/lib/vis/configuration';

export type TableProps = {
  showBarplot: boolean;
  itemsPerPage: number;
  displayAttributes: string[];
};

const configuration: TableProps = {
  itemsPerPage: 10,
  showBarplot: false,
  displayAttributes: [],
};

export const TableVis = ({ data, schema, configuration }: VisualizationPropTypes) => {
  const ref = useRef<HTMLDivElement>(null);
  console.log(configuration);
  const attributesArray = useMemo<AugmentedNodeAttributes[]>(
    () =>
      data.nodes.map((node) => {
        const types: SchemaAttribute[] =
          schema.nodes.find((n) => n.key === node.label)?.attributes?.attributes ??
          schema.edges.find((r) => r.key === node.label)?.attributes?.attributes ??
          [];

        return {
          attribute: node.attributes,
          type: Object.fromEntries(types.map((t) => [t.name, t.type])),
        };
      }),
    [data.nodes],
  );

  return (
    <div className="h-full w-full" ref={ref}>
      {attributesArray.length > 0 && (
        <Table
          data={attributesArray}
          itemsPerPage={configuration.itemsPerPage}
          showBarPlot={configuration.showBarplot}
          showAttributes={configuration.displayAttributes}
        />
      )}
    </div>
  );
};

const TableSettings = ({
  configuration,
  graph,
  updateSettings,
}: {
  configuration: TableProps;
  graph: GraphMetaData;
  updateSettings: (val: any) => void;
}) => {
  const allAttributes: string[] = Object.keys(graph.nodes.types).reduce((acc: string[], label: string) => {
    const labelAttributes = Object.keys(graph.nodes.types[label].attributes);
    return acc.concat(labelAttributes);
  }, []);

  // Find the intersection of attributes across all nodes
  const intersectionAttributes = allAttributes.filter((attr) => {
    return Object.keys(graph.nodes.types).every((label) => {
      return graph.nodes.types[label].attributes.hasOwnProperty(attr);
    });
  });

  const uniqueIntersectionAttributes = Array.from(new Set(intersectionAttributes));

  return (
    <SettingsContainer>
      <Input
        type="dropdown"
        label="Items per page"
        value={configuration.itemsPerPage}
        onChange={(val) => updateSettings({ itemsPerPage: val })}
        options={[10, 20, 30]}
      />
      <Input
        type="boolean"
        label="Show barplot"
        value={configuration.showBarplot}
        onChange={(val) => updateSettings({ showBarplot: val })}
      />

      <div>
        <span className="text-sm">Attributes to display</span>
        <div className="h-44 overflow-y-auto">
          <Input
            type="checkbox"
            value={configuration.displayAttributes}
            options={uniqueIntersectionAttributes}
            onChange={(val: string[] | string) => {
              const updatedVal = Array.isArray(val) ? val : [val];
              updateSettings({ displayAttributes: updatedVal });
            }}
          />
        </div>
      </div>
    </SettingsContainer>
  );
};

export const TableComponent: VISComponentType = {
  displayName: 'TableVis',
  component: TableVis,
  settings: TableSettings,
  configuration: configuration,
};

export default TableComponent;
